/*
 * FUSD - The Framework for UserSpace Devices - Example program
 *
 * Jeremy Elson <jelson@circlemud.org>
 *
 * hello-world: Simply creates a device called /dev/hello-world, which
 * greets you when you try to get it.
 *
 * $Id: helloworld.c,v 1.9 2001/09/02 20:05:00 jelson Exp $
 */

/* EXAMPLE START helloworld.c */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "fusd.h"

#define GREETING "Hello, world!\n"

int do_open_or_close(struct fusd_file_info *file)
{
  return 0; /* attempts to open and close this file always succeed */
}

int do_read(struct fusd_file_info *file, char *user_buffer, 
	    size_t user_length, loff_t *offset)
{
  int retval = 0;
  
  /* The first read to the device returns a greeting.  The second read
   * returns EOF. */
  if (*offset == 0) {
    if (user_length < strlen(GREETING))
      retval = -EINVAL;  /* the user must supply a big enough buffer! */
    else {
      memcpy(user_buffer, GREETING, strlen(GREETING)); /* greet user */
      retval = strlen(GREETING); /* retval = number of bytes returned */
      *offset += retval; /* advance user's file pointer */
    }
  }
  
  return retval;
}

int main(int argc, char *argv[])
{
  struct fusd_file_operations fops = {
    open: do_open_or_close,
    read: do_read,
    close: do_open_or_close };
  
  if (fusd_register("/dev/hello-world", 0666, NULL, &fops) < 0)
    perror("Unable to register device");
  else {
    printf("/dev/hello-world should now exist - calling fusd_run...\n");
    fusd_run();
  }
  return 0;
}
/* EXAMPLE STOP helloworld.c */
