/*
 * /proc/zero test program
 *
 * $Id: zero.c,v 1.3 2001/03/03 01:27:25 cvs Exp $
 */


#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/tqueue.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#ifdef MODULE
#include <linux/module.h>
#endif
#include <linux/timer.h>
#include <linux/delay.h>
#include <linux/errno.h>
#include <linux/malloc.h>
#include <linux/types.h>
#include <linux/fs.h>
#include <linux/poll.h>
#include <linux/devfs_fs_kernel.h>

#ifndef MODULE
#define MOD_INC_USE_COUNT
#define MOD_DEC_USE_COUNT
#endif

/************************************************************************/

static ssize_t zero_read(struct file *file,
			 char *user_buffer,    /* The buffer to fill with data */
			 size_t user_length,   /* The length of the buffer */
			 loff_t *offset)  /* Our offset in the file */
{
  int retval = 0;
  char *buffer;
  buffer = kmalloc(user_length, GFP_KERNEL);
  if (buffer == NULL) {
    return -ENOMEM;
  }

  memset(buffer, 0, user_length);

  if (copy_to_user(user_buffer, buffer, user_length)) {
    retval = -EFAULT;
    goto out;
  }

  retval = user_length;
  
 out:
  kfree(buffer);
  return retval;
}


static unsigned int zero_poll(struct file *file, poll_table *wait)
{
  return POLLIN | POLLRDNORM;
}


static int zero_open(struct inode *inode, struct file *file)
{
  MOD_INC_USE_COUNT;
  return 0;
}


static int zero_release(struct inode *inode, struct file *file)
{
  MOD_DEC_USE_COUNT;
  return 0;
}


static struct file_operations zero_fops = {
  owner: THIS_MODULE,
  read: zero_read,
  poll: zero_poll,
  open: zero_open,
  release: zero_release
};


static devfs_handle_t zero_handle = 0;

int init_module(void)
{
  zero_handle = devfs_register(NULL, "zero2",
			       DEVFS_FL_AUTO_DEVNUM,
			       0, 0,
			       S_IFCHR | S_IRUSR | S_IWUSR, //| 0666,
			       &zero_fops, NULL);

  if (zero_handle == 0) {
    printk("zero2: unable to register character device\n");
    return -EIO;
  }

  return 0;
}



void cleanup_module(void)
{
  /* unregister the character device */
  devfs_unregister(zero_handle);
  printk("done..\n");
}



