/*

  opentest.c 

  tests open, close call

 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#include "fusd.h"

int testnums[5] = {12,34,56,78,0};
int counter = 0;

int _open(struct fusd_file_info *file)
{
  if (0 && (counter == 0)) {
    fusd_return(file, -testnums[0]);
    counter++;
    return -FUSD_NOREPLY;
  }
  
  else
    return -testnums[counter++];
}


int _close(struct fusd_file_info *file)
{
  if (counter == 5) {
    fusd_return(file, -testnums[0]);
    counter++;
    return -FUSD_NOREPLY;
  }
  
  else
    return -testnums[counter++ - 5];
}



int main(int argc, char *argv[])
{
  int f;

  if (argc != 2) {
    fprintf(stderr, "usage: %s <device-to-create>\n", argv[0]);
    exit(0);
  }

  /* fork into client and server */
  f = fork();
  if (f<0) {
    perror("Fork failed");
    exit(1);
  }

  /* server */
  if (f) {
    // init function table
    fusd_file_operations_t
      fops = {_open, 
	      _close,
	      NULL,
	      NULL,
	      NULL,
	      NULL,
	      NULL};
    
    int fd;
    if ((fd = fusd_register(argv[1], 0666, 0, &fops)) < 0) {
      perror("Register failed");
    } 
    
    else {
      //fusd_run(fd, 3);
      while (1) fusd_dispatch(fd);
    }
  }

  /* client */
  else {
    int i;
    int fd;
    char devname[20];
    int error = 0;
    int j;

    sleep(1);

    sprintf(devname, "/dev/%s", argv[1]);

    /* opens */
    for (i=0; i<5; i++) {
      fd = open(devname, O_RDWR);

      if (i != 4) {
	fprintf(stderr, "Open call returned %d, should be %d.\n",
		errno, testnums[i]);
	if (errno != testnums[i])
	  error = 1;
      }
      else {
	fprintf(stderr, "Open call return fd = %d, should be positive.\n", fd);
	if (fd < 0) error = 1;
      }
      
    }

    /* close */
    j = close(fd);
    fprintf(stderr, "Close call returned %d, should be 0.\n", j);
    if (j != 0) error = 1;

    close(fd);
    fprintf(stderr, "Close call returned %d, should be 9.\n", errno);
    if (errno != 9) error = 1;
    
    if (error)
      exit(1);
  }

  exit(0);
}


